--=========================================================
--
-- フェードエフェクト：
-- ずんだ餅のカーテン
--
--=========================================================
Fade_zunda = {}
Fade_zunda.new = function ()
	
	local this = {}
	
	this.fadeRate = 0.0
	this.phase = ""
	
	-- 描画時の位置調整用にウィンドウのサイズを確保しておく
	this.windowWidth, this.windowHeight, this.windowFlags = love.window.getMode( )
	
	-- ずんだ餅の画像
	this.image = love.graphics.newImage ( "zunda.png" )
	
	this.zundaLines = {}
	
	--=========================================================
	-- 開始
	--=========================================================
	this.start = function(self)
		
		-- 乱数初期化。各列ごとに、ずんだ餅の位置を微妙にランダムにズラす。
		math.randomseed(os.time())
		
		self.fadeRate = 0.0
		self.phase = "out"
		self.zundaLines = {}
		
		local zundaLineLength = 6    -- ずんだ餅の列の数
		local zundaRowDensity = 180  -- 横並びの距離。小さいほど餅の数が増える。
		
		local zundaW = this.image:getWidth()
		local zundaH = this.image:getHeight()
		
		for y = 1, zundaLineLength, 1 do
			
			local zundaLine = {}
			for x = 0, self.windowWidth, zundaRowDensity do
				
				local zundaData = {
					
					y = -zundaH,
					x = x,
				}
				
				zundaLine [ #zundaLine + 1 ] = zundaData
				
			end

			-- シャッフル
			for i = #zundaLine, 1, -1 do
				
				local j = math.random(1,i)
				
				local temp = zundaLine[i]
				zundaLine[i] = zundaLine[j]
				zundaLine[j] = temp

			end
			
			self.zundaLines [ #self.zundaLines + 1 ] = zundaLine
		end
		
		
	end
	
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self)
		
		local pace = 0.03
		
		-- フェードアウト
		if ( self.phase == "out" ) then
			
			self.fadeRate = self.fadeRate + pace
			
			if ( self.fadeRate >= 1.0 ) then
				self.fadeRate = 1.0
				self.phase = "in"
				
				self:changeScene()
			end
			
		-- フェードイン
		elseif ( self.phase == "in" ) then
			
			self.fadeRate = self.fadeRate - pace
			
			if ( self.fadeRate <= 0.0 ) then
				self.fadeRate = 0.0
				self.phase = ""
			end
			
		end
		
	end
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		-- 呼び出し元に影響が出ないよう、この時点での描画色を確保（後で戻す）
		local r,g,b,a = love.graphics.getColor()
		
		love.graphics.setColor(1,1,1,1) -- ここでの描画色（白：画像の色そのまま）
		
		-- 表示位置の調整のため、実際の進行度に少しゲタを履かせる（適当）
		local progress = self.fadeRate * 1.2
		if ( progress > 1.0 ) then progress = 1.0 end
		
		-- 列と列の距離（フェードの進行度が目一杯になってカーテンが完全に閉まった時点での距離）
		local lineDistance = math.floor ( (self.windowHeight * 1.03) / #self.zundaLines)
		
		for y = 1, #self.zundaLines, 1 do
			
			-- カーテンが閉まり切った場合の、この列のY位置。ここにフェードの進行度を掛ける。
			local goalY = lineDistance * y
			
			local zundaLine = self.zundaLines[y]
			
			for x = 1, #zundaLine, 1 do
				
				love.graphics.draw ( 
					self.image, 
					zundaLine[x].x, 
					goalY * progress, -- Y位置。カーテンが閉まり切った場合の位置を基準に、進行度に応じて調整。
					0,1,1,
					self.image:getWidth()/2,
					self.image:getHeight()
				)
				
			end
		end
		
		-- 描画色を戻す
		love.graphics.setColor(r,g,b,a)
		
	end
	
	
	
	return this
end









